function [s_clk,s_match] = DPLLMnMClockRecovery(s_rx,sps,Gp,Gi,rx_filter,span,shape,beta)
%   This function recovers the clock/timing of an FM0 signal using 
%   a digital phase-locked loop (DPLL). The timing error is estimated 
%   using the Mueller & Muller method. For details on this method refer to: 
%   K. Mueller and M. Muller, "Timing Recovery in Digital Synchronous Data Receivers," 
%   in IEEE Transactions on Communications, vol. 24, no. 5, pp. 516-531, May 1976. 
%   and G. Danesfahani and T. Jeans, Optimisation of modified Mueller and Muller algorithm,
%   in Electron. Lett., vol. 31, no. 13, pp. 10321033, Jun. 1995.
%  
% Inputs:
%   s_rx - transponder FM0 signal to be demodulated
%   sps - nominal number of samples per FM0 symbol
%   Gp - proportional gain of PLL loop filter 
%   Gi - integral gain of PLL loop filter
%   rx_filter - receiver matched filter (0: rectangular pulse, 1: raised cosine)
%   span - filter span in symbols (raised cosine only)
%   shape - filter shape (raised cosine only)
%   beta - filter roll-off (raised cosine only)
% Outputs:
%  s_matched - matched filter output
%  s_clk - decimated clock signal containing two BPSK pulses per FM0 symbol
% Author: Alirio Boaventura

    s_match = [];       % Initialize matched filter output
    s_clk = [];         % Initialize clock signal
    Tsymb = sps/2;      % Nominal symbol interpolation interval considering that 
                        % each FM0 symbol contains two BPSK symbols with duration sps/2 
    rPrev = 0;          % Matched filter output at previous interpolation instant r(k-1)
    rCurrent = 0;       % Matched filter output at current interpolation instant r(k)
    aPrev = 0;          % Decision on matched filter output at previous interpolation instant a(k-1)
    aCurrent = 0;       % Decision on matched filter output at current interpolation instant a(k)
    cumulError = 0;     % Cumulative error cum_e(k)
    error = 0;          % Timing error e(k)
    tau = 0;            % Timing offset t(k)
    Tinterp = 0;        % Symbol interpolation instant

    % Design receiver matched filter 
    % In general, the receiver matched impulse response should be a time-reversed 
    % version of the impulse response of the transmit pulse-shape filter, but     
    % since the transmit pulse is symmetric, there's no need for time flipping 
    if (rx_filter == 0)     % Rectangular pulse shape
        b_match = ones(1,Tsymb);
    elseif (rx_filter == 1) % Raised cosine filter
        % Any raised cosine filter design function   
        % can be used to implement this filter
        rxfilter = comm.RaisedCosineReceiveFilter(...
        'Shape',shape,'RolloffFactor',beta,...
        'FilterSpanInSymbols',span,...
        'InputSamplesPerSymbol',sps/2,...
        'DecimationFactor',1);  % We do not decimate the received signal here 
                                % Decimation is performed within the clock recovery loop
        b_match = coeffs(rxfilter).Numerator;
    end
    % Compute matched filter output
    if(rx_filter == 0 || rx_filter == 1)
        s_match = filter(b_match,1,s_rx);
    else
        s_match = s_rx;
    end
    s_match = s_match./max(s_match);   % normalize
    % Clock recovery loop
    for k = 1:round(length(s_rx)/Tsymb)  
        % Interpolation controller
        Tinterp = k*Tsymb+tau;      % Update interpolation sampling time 
        tBase = floor(Tinterp);     % Compute interpolation basepoint 
        tFrac = Tinterp-tBase;      % Compute interpolation fractional interval 
        
        % Interpolate matched filter output
        if(tBase>0 && tBase<length(s_match)-1)                    % Boundary check    
            rCurrent = Interpolate(s_match(tBase),s_match(tBase+1),tFrac);    % Linear interpolation
        else
            rCurrent = 0;
        end
        
        % Apply hard decision to interpolated matched filter output 
        s_clk(k) = 2*(rCurrent > 0)-1;

        % Save decimated sample
        aCurrent = s_clk(k);
        % Estimate timing error using Mueller and Muller method
        error = (3*Tsymb/16)*(rCurrent*aPrev-rPrev*aCurrent);
        
        % Compute timing offset 
        tau = (tau+Gp*error+Gi*cumulError)*1e-3; 

        % Save current matched filter output and interpolated decision
        rPrev = rCurrent;
        aPrev = aCurrent;  
        
        % Update cumulative error
        cumulError = cumulError+error;  
    end  
 end

