function [bin_data,s_tag,s_cod] = GenRandFM0Frame_2(sps,nSymb,snrdB,tx_filter,dc_offset,phase_shift,beta,span,shape)
% This function generates a BPSK-modulated waveform containing random FM0-encoded transponder data  
% Input:
%   sps - number of samples per FM0 symbol
%   nSymb - number of FM0 symbols
%   snrdB - signal-to-noise ratio in dB (Note: for BPSK, SNR=EbN0=EsN0)
%   tx_filter - transmit filter (0: rectangular 1: raised cosine)
%   dc_offset - complex baseband DC offset
%   phase_shift - constellation phase rotation
%   beta - filter roll-off (raised cosine only)
%   span - filter span in symbols (raised cosine only)
%   shape - filter shape (raised cosine only)
% Outputs:
%   data -  random binary transponder data
%   s_tag -  transponder FM0 waveform
% Author: Alirio Boaventura

% Generate random binary data
bin_data = randi([0 1],1,nSymb);
bin_data(1) = 0;  % force first bit to zero

% FM0 symbol 0 basis pulse  
s0 = [1 -1];
% FM0 symbol 1 basis pulse   
s1 = [1 1];

% Pilot tone
pilot = s0;
for n = 1:11
    pilot = [pilot,s0];
end

% Preamble pattern
%preamb = [s1 -s0 -s1 s0 -s1 s1];   % Original FM0 preamble pattern
preamb = [-s0 -s1 s0 -s1];       % Shorter FM0 preamble pattern

% FM0 header 
s_cod = [pilot,preamb,-s0];

% FM0 biorthogonal encoding
last_was_pos = 1;
for n = 2:nSymb
    if (last_was_pos == 1)
        if (bin_data(n) == 0)
           s_cod = [s_cod,-s0]; 
        else
           s_cod = [s_cod,-s1];
           last_was_pos = 0;
        end
    else
        if (bin_data(n) == 0)
           s_cod = [s_cod,s0]; 
        else
           s_cod = [s_cod,s1];
           last_was_pos = 1;
        end
    end
end
% Force a final transition with a symbol '1'
s_cod = [s_cod -s_cod(end) -s_cod(end)];
% Termination sequence to compansate for transmit-receive filer delay
s_cod = [s_cod, s_cod(end)*ones(1,4*span)];

% Interpolation (upsampling followed by pulse-shape filtering)
% The signal is upsampled at half the desired oversampling because 
% each FM0 symbol has been pre-coded with two binary symbols
% (i.e., it is already upsampled by a factor of 2)
if(tx_filter == 1)    % interpolation using raised cosine filter
    % Note that other raised cosine filter design tools could be used to  
    % implement this filter (in case the communications toolbox unavailable
    txfilter = comm.RaisedCosineTransmitFilter(...
        'OutputSamplesPerSymbol',sps/2,...
        'RolloffFactor',beta,'Shape',shape,...
        'FilterSpanInSymbols',span');
    s_shaped = txfilter(s_cod.'); 
%     % Compensate filter delay
%     delay = mean(grpdelay(txfilter));
%     s_filt(1:delay) = [];
    
else  % interpolation using rectangular pulse shape
    s_shaped = [];
    for n = 1:length(s_cod)
        s_shaped = [s_shaped ones(1,sps/2)*s_cod(n)];
    end  
end
    % Create complex baseband signal
    s_mod = complex(s_shaped,0).'; 
    % Pass signal through AWGN cahnnel with prescribed SNR
    s_awgn = awgn(s_mod,snrdB,'measured');
    % Apply phase rotation and complex DC offset
    s_tag = s_awgn*exp(j*deg2rad(phase_shift))+dc_offset;     
end

