function [data,s_tag] = GenRandQPSKFrame(sps,nSymb,mOrder,snr_dB,tx_filter,dc_offset,phase_shift,phase_sort,beta,span,shape)
% This function generates a QPSK signal with variable parameters modulted by random data 
% Input:
%   sps - number of samples per FM0 symbol
%   nSymb - number of FM0 symbols
%   mOrder - QPSK modulation order
%   snr - signal-to-noise ratio (in dB)
%   tx_filter - transmit filter (0: rectangular, 1: raised cosine)
%   dc_offset - complex baseband DC offset (in Volt)
%   phase_shift - constellation phase rotation (in Degree)
%   phase_sort - phase sorting (0: no phase sorting, 1: sort QPSK phases)
%   beta - filter roll-off (raised cosine only)
%   span - filter span in symbols (raised cosine only)
%   shape - filter shape (raised cosine only)
% Outputs:
%   data -  random binary transponder data
%   s_tag -  transponder FM0 waveform
% Author: Alirio Boaventura

% Generate random binary data
data = randi([0 1],1,nSymb);

% Check that mOrder is a positive integer of the form 2^n, 
% with n=1,2,3,... 
if (~isreal(mOrder) || ~isscalar(mOrder) || mOrder<=0 ...
    || (ceil(mOrder)~=mOrder) || ~isnumeric(mOrder) || ... 
    ~isnumeric(mOrder) || (ceil(log2(mOrder)) ~= log2(mOrder)) ...
    || (min(min(data)) < 0) || (max(max(data)) > (mOrder-1)))
    error('mOrder must be a positive integer of the form 2^n, n=1,2,3,...');
end

% Map binary data into QPSK symbols
N = length(data);
M = floor(N/mOrder);
% Zero padding to ensure an integer number of QPSK symbols
if(mod(N,mOrder) ~= 0) 
  R = N-mOrder*M;
  data = [data zeros(1,mOrder-R)];
end
N = length(data);
% Convert binary data into decimal symbols
s_dec_symb = [];
for n = 1:N/log2(mOrder)
    s_dec_symb(n) = bi2de(data((n-1)*log2(mOrder)+1:n*log2(mOrder)),'left-msb');
end

%s_dec_symb=randi([0 mOrder-1],nSymb,1);
% Compute QPSK sumbol phase based on mOrder. The phase lies between 0 and 2*pi. 
theta = 2*pi*s_dec_symb/mOrder;
% Sort phases
if(phase_sort == 1)
     theta=sort(theta);
end
% Convert decimal symbols into QPSK symbols
% The complex envelope can be expressed as exp(j*theta)
s_mod_qpsk = exp(j*theta);

% Upsample/Interpolate
if(tx_filter == 1)    % interpolate with a raised-cosine filter
    % Filter the signal through a transmit RRC filter
    txfilter = comm.RaisedCosineTransmitFilter(...
        'OutputSamplesPerSymbol',sps,...
        'FilterSpanInSymbols',span,...
        'Shape',shape,'RolloffFactor',beta);
    s_filt = txfilter(s_mod_qpsk.'); 
else % use rectangular pulse shaping filter
    s_filt = [];
    for n = 1:length(s_mod_qpsk)
        s_filt = [s_filt ones(1,sps)*s_mod_qpsk(n)];
    end  
end
    % Add AWGN with prescribed signal-to-noise ratio
    s_awgn = awgn(s_filt,snr_dB,'measured');
    % Apply phase rotation and complex DC offset
    s_tag = s_awgn*exp(j*deg2rad(phase_shift))+dc_offset;   
end



