function [data] = ZCrossFM0Demod(s_rx,sps,nSymbs)
%   This function demodulates an FM0 signal using zero crossing detection.
%   The signal timing is first recovered by counting the number of samples between 
%   consecutive signal transitions and the data is then decoded by evaluating the 
%   stored lengths using a pre-defined threshold.
%
%   Input:
%       s_rx - received FM0 signal
%       sps - number of samples per symbol
%       nSymb - expected number of symbols
%   Output:
%       data - decoded binary data
%   Author: Alirio Boaventura
    
    N = length(s_rx);
    data = zeros(1,nSymbs);         % Initialize output data vector 
    tol = 0.2;                      % Allowed phase deviation relative to nominal sps (20%)
    last_was_pos = 0;               % Sign of last processed sample
    nCrosses = 0;                   % Number of zero crossings
    nSamps = 0;                     % Number of samples processed since last zero crossing
    MaxIters = 100;                 % Max number of iterations searching for preamble
    d_v_max = 1.5*sps*(1 + tol);    % Max duration of FM0 symbol 'V' 
    d_v_min = 1.5*sps*(1 - tol);    % Min duration of FM0 symbol 'V' 
    d_1_max = sps*(1 + tol);        % Max duration of FM0 symbol '1'
    d_1_min = sps*(1 - tol);        % Min duration of FM0 symbol '1'
    d_half0_max = sps*(1 + tol)/2;  % Max duration of half FM0 symbol '0'
    d_half0_min = sps*(1 - tol)/2;  % Min duration of half FM0 symbol '0'
    clk = [];
    %% Zero-crossing detection and length storage (timing recovery)
    for n = 1:N
        nSamps = nSamps+1;
        % Negative and positive going zero crossings are detected when x(n)*x(n+1) < 0
        if((last_was_pos && ~IsPositive(s_rx(n))) || (~last_was_pos && IsPositive(s_rx(n)))) 
            nCrosses = nCrosses+1;      % Increment number of zero crossings detected
            clk(nCrosses) = nSamps;    % Save distance between consecutive zero crossings
            nSamps = 0;              % Reset number of samples since last zero crossing
        end
        last_was_pos = IsPositive(s_rx(n));
    end
    
    %% Preamble detection
    M = length(clk);
    n = 1;
    while(n < MaxIters)
        if(n+6 > M)
            break
        end
        P1 = ((clk(n) > d_1_min) && (clk(n) < d_1_max));                % Symbol '1'
        P2 = ((clk(n+1) > d_half0_min) && (clk(n+1) < d_half0_max));    % Half symbol '0' 
        P3 = ((clk(n+2) > d_half0_min) && (clk(n+2) < d_half0_max));    % Half symbol '0' 
        P4 = ((clk(n+3) > d_1_min) && (clk(n+3) < d_1_max));            % Symbol '1' 
        P5 = ((clk(n+4) > d_half0_min) && (clk(n+4) < d_half0_max));    % Half symbol '0' 
        P6 = ((clk(n+5) > d_v_min) && (clk(n+5) < d_v_max));            % Symbol 'V' 
        P7 = ((clk(n+6) > d_1_min) && (clk(n+6) < d_1_max));            % Symbol '1' 
        if(P1 && P2 && P3 && P4 && P5 && P6 && P7)  % Check for occurrence of preamble pattern
            n = n+7;  % Starting position of payload passed to the following loop 
            break;
        end
        n = n+1;
    end
    %% Threshold decision and symbol demapping
    for m = 1:nSymbs
       if(n > M)
           break;
       end
       % Symbol '1' is detected if (1-tol)*sps < clk(n) < (1+tol)*sps
       if((clk(n) > sps*(1-tol)) && (clk(n) < sps*(1+tol)))
           data(m) = 1;
           n = n+1;
       % Symbol '0' is detected otherwise
       else
           data(m) = 0;
           n = n+2;   % Increment twice as symbol '0' has two transitions
       end
    end
end

