function [s_tx] = EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm)
%   This function encodes, modulates and interpolates reader command data
%   The data are encoded using pulse interval encoding (PIE) and modulated 
%   using ASK, SSB-ASK or PR-ASK. Interpolation and pulse-shaping are 
%   performed in a signle step using a rectangular or raised cosine filter
%   Inputs:
%       data - binary payload data of the reader command to be transmitted
%       mod_type - modulation type (0: DSB-ASK, 1: PR-ASK, 2: SSB-ASK)
%       command_type - command type (1: Query, 0: all other commands)
%       beta - filter roll-off (raised cosine only)
%       tari - duration of PIE symbol '0'
%       tx_filter - defines transmit filter (0: rectangular pulse, 1: raised cosine)
%       span - filter span in symbols (raised cosine only)
%       shape - filter shape (raised cosine only)
%       norm - nomalize filter gain (raised cosine only)
%   Output:
%       s_tx - reader transmit baseband waveform 
%   Author: Alirio Boaventura

if(mod_type == 2 && tx_filter == 0)
    error('SSB modulation not allowed with rectangular pulse shape');
end

d_0 = tari;                 % Duration of PIE symbol '0'
d_pw = tari/2;              % Duration of PIE pulse (PW) 
d_1 = 2*tari;               % Duration of PIE symbol '1'
d_av = (d_0+d_1)/2;         % Average transmit symbol duration
tx_dr = 1/d_av;             % Average transmit data rate
Fs = 2.56e6;                % Sampling rate (2.56 Megasample/s)
Ts = 1/Fs;                  % Sampling period
n_0 = round(d_0/Ts);        % Number of samples of symbol '0'
n_1 = round(d_1/Ts);        % Number of samples of symbol '1'
n_pw = round(d_pw/Ts);      % Number of samples of PIE pulse
upfac = n_pw;               % Upsampling factor
s_re = 0;                   % Real part of the signal
s_im = 0;                   % Imaginary part of the signal

% PIE encoding
% Symbol '0' and '1' contain 2 and 4 pulses of duration tari/2,
% respectively. Each symbol will be upsampled by a factor of upfac = 8
s0 = [1 0];         % Symbol '0' (duration of tari)
s1 = [1 1 1 0];     % Symbol '1' (duration of 2*Tari)

% Define Preamble and FrameSync
d_delim = 12.5e-6;              % Duration of delimiter symbol
d_RTcal = d_0+d_1;              % Duration of reader-to-tag calibration symbol (d_0+d_1)   
d_TRcal = 37.5e-6;              % Duration of tag-to-reader calibration symbol 
                                % The d_TRcal and the DR parameter in the query command 
                                % define the transponder's BLF (see ISO 18000-63, table 6.9) 
                                % (e.g., DR = 64/8 and d_TRcal = 33.3e-6 -> max BLF = 640kHz) 
n_delim = round(d_delim/Ts);    % Number of (low) samples of the delimiter
n_RTcal = round(d_RTcal/Ts);    % Number of (high) samples of RTcal
n_TRcal = round(d_TRcal/Ts);    % Number of (high) samples of TRcal
preamb_dsb_ask = [zeros(1,n_delim/n_pw), s0, ones(1,n_RTcal/n_pw), ...
    zeros(1,n_pw/n_pw), ones(1,n_TRcal/n_pw), zeros(1,n_pw/n_pw)];  % Preamble DSB-ASK
preamb_pr_ask = [zeros(1,n_delim/n_pw), s0, -ones(1,n_RTcal/n_pw), ...
    zeros(1,n_pw/n_pw), ones(1,n_TRcal/n_pw), zeros(1,n_pw/n_pw)];  % Preamble PR-ASK
% DSB-ASK FrameSync
frameSync_dsb_ask = [zeros(1,n_delim/n_pw), s0, ones(1,n_RTcal/n_pw), zeros(1,n_pw/n_pw)];  
% PR-ASK Frame-sync
frameSync_pr_ask = [zeros(1,n_delim/n_pw), s0, -ones(1,n_RTcal/n_pw), zeros(1,n_pw/n_pw)];       

% Symbol mapping/Modulation (DSB-ASK, SSB-ASK or PR-ASK)
invert_next = 1;
if (mod_type == 0 || mod_type == 2) % DSB-ASK or SSB-ASK 
    if (command_type == 1)          % Start with Preamble 
        s_re = [preamb_dsb_ask];
    else                            % Start with FrameSync 
        s_re = [frameSync_dsb_ask];
    end
    for n = 1:length(data)
        if (data(n) == 1)           % Add symbol '1'
            s_re = [s_re,s1];
        else                        % Add symbol '0'
            s_re = [s_re,s0];
        end
    end
elseif(mod_type == 1)               % PR-ASK
    if (command_type == 1)
        s_re = [preamb_pr_ask];    % Start with Preamble
        invert_next = 1;
    else 
        s_re = [frameSync_pr_ask]; % Start with FrameSync
        invert_next = 0;
    end
    for n = 1:length(data)
        if (data(n) == 1)           % Add symbol '1'
            if(invert_next == 1)
                s_re = [s_re,-s1];
                invert_next = 0;
            else
                s_re = [s_re,s1];
                invert_next = 1;
            end
        else                        % Add symbol '0'
            if (invert_next == 1)   % Invert next symbol
                s_re = [s_re,-s0]; 
                invert_next = 0;
            else                     % No symbol inversion
                s_re = [s_re,s0];
                invert_next = 1;
            end
        end
    end
end

% Interpolation (upsamplig and pulse-shape filtering)
if(tx_filter == 0)      % Rectangular pulse shape
    s_re = rectpulse(s_re,upfac);
elseif(tx_filter == 1)  % Raised cosine filter
    
    txfilter  =  comm.RaisedCosineTransmitFilter(...
          'OutputSamplesPerSymbol',upfac,...
          'Shape',shape,'RolloffFactor',beta,...
          'FilterSpanInSymbols',span);
    if(norm == 1)
        % Normalize filter to obtain maximum filter tap value of 1
        coef = coeffs(txfilter);
        txfilter.Gain = 1/max(coef.Numerator);
    end
    s_re = txfilter(s_re.');
    % Compensate for filter delay
    delay = mean(grpdelay(txfilter));
    s_re(1:delay) = [];
end

if (mod_type == 2)          % Create complex SSB-ASK signal using Hilbert transform
    N = 40;                 % Filter order
    delay = N/2;            % Filter delay
    trans_band = 400e3;     % Transition bandwidth
    Hilbert = designfilt('hilbertfir','FilterOrder',N, ...
       'TransitionWidth',trans_band,'SampleRate',Fs,'DesignMethod','equiripple');
    s_im = filter(Hilbert,s_re);
    % compensate for the filter delay
    s_tx = complex(s_re(1:end-delay),s_im(delay+1:end));
else    % Generate complex DSB-ASK or PR-ASK signal    
    s_tx = complex(s_re,s_im);    
end
end